@@ -32,7 +32,7 @@ def _get_openssl_libraries(platform):
         # In some circumstances, the order in which these libs are
         # specified on the linker command-line is significant;
         # libssl must come before libcrypto
-        # (http://marc.info/?l=openssl-users&m=135361825921871)
+        # (https://marc.info/?l=openssl-users&m=135361825921871)
         return ["ssl", "crypto"]
 
 
@@ -46,7 +46,7 @@ def _extra_compile_args(platform):
     When we drop support for CRYPTOGRAPHY_OPENSSL_LESS_THAN_110 we can
     revisit this.
     """
-    if platform != "win32":
+    if platform not in ["win32", "hp-ux11", "sunos5"]:
         return ["-Wconversion", "-Wno-error=sign-conversion"]
     else:
         return []
@@ -64,7 +64,6 @@ ffi = build_ffi_for_binding(
         "bignum",
         "bio",
         "cmac",
-        "cms",
         "conf",
         "crypto",
         "ct",
