@@ -7,18 +7,10 @@ from __future__ import absolute_import, division, print_function
 import pytest
 
 from cryptography.hazmat.primitives.asymmetric.utils import (
-    Prehashed, decode_dss_signature, decode_rfc6979_signature,
-    encode_dss_signature, encode_rfc6979_signature,
+    Prehashed, decode_dss_signature, encode_dss_signature
 )
 
 
-def test_deprecated_rfc6979_signature():
-    sig = pytest.deprecated_call(encode_rfc6979_signature, 1, 1)
-    assert sig == b"0\x06\x02\x01\x01\x02\x01\x01"
-    decoded = pytest.deprecated_call(decode_rfc6979_signature, sig)
-    assert decoded == (1, 1)
-
-
 def test_dss_signature():
     sig = encode_dss_signature(1, 1)
     assert sig == b"0\x06\x02\x01\x01\x02\x01\x01"
