@@ -16,7 +16,7 @@ from ...utils import load_nist_kbkdf_vectors
 
 @pytest.mark.requires_backend_interface(interface=HMACBackend)
 class TestCounterKDFCounterMode(object):
-    test_HKDFSHA1 = generate_kbkdf_counter_mode_test(
+    test_kbkdfctr = generate_kbkdf_counter_mode_test(
         load_nist_kbkdf_vectors,
         os.path.join("KDF"),
         ["nist-800-108-KBKDF-CTR.txt"]
